/* $Id: main.c,v 1.3 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#define _GNU_SOURCE
#define _USE_GNU

#include "defs.h"
#include "rom.h"
#include "pio.h"
#include "vram.h"
#include "card.h"
#include "error.h"
#if defined(linux)
#include <sys/io.h>
#endif
#if defined(__NetBSD__)
#include <machine/sysarch.h>
#endif

/* GNU long-options */

enum {
	OPTION_PC		= 'P' ,			// this section defines the shortcuts
	OPTION_GP		= 'G' ,
	OPTION_SP		= 'S' ,
	OPTION_TCB		= 't' ,
	OPTION_EVENT	= 'e' ,
	OPTION_GO		= 'g' ,
	OPTION_CONSOLE	= 'o' ,
	OPTION_ADDRESS	= 'a' ,
	OPTION_LENGTH	= 'n' ,
	OPTION_UPLOAD	= 'u' ,
	OPTION_DOWNLOAD	= 'd' ,
	OPTION_PORT		= 'p' ,
	OPTION_REALTIME	= 'r' ,
	OPTION_QUIET	= 'q' ,
	OPTION_HELP		= 'h' ,
	OPTION_VERBOSE	= 'v' ,
	OPTION_VERSION	= 'V' ,
	OPTION_INFO		= 'i' ,
	OPTION_SLOT		= 's' ,
	OPTION_BACKUP	= 'B' ,
	OPTION_RESTORE	= 'R' ,
	OPTION_LIST		= 'l' ,
	OPTION_FORMAT	= 'F' ,
	OPTION_DELETE	= 'D' ,
	OPTION_KANJI	= 'k' ,
	OPTION_VRAM		= 'I' ,
	OPTION_LEFT		= 'X' ,
	OPTION_TOP		= 'Y' ,
	OPTION_WIDTH	= 'W' ,
	OPTION_HEIGHT	= 'H' ,
	OPTION_SCRIPT	= '@' ,
	OPTION_EXEC		= 'x' ,
	OPTION_FLASH	= 'f' ,
	OPTION_BOOT		= '!' ,
	OPTION_NICE		= 'N' ,
	OPTION_BPP		= 'b' ,
	OPTION_RESET		= 'z' ,
	OPTION_CALL		= 'c' ,
	OPTION_LIBPS	= 'L' ,
	OPTION_YAROZE	= 'y' ,
	OPTION_CHROOT	= 'T' ,
};

static struct option long_options[] =
{
	{ "pc" ,required_argument, NULL, OPTION_PC },
	{ "gp" ,required_argument, NULL, OPTION_GP },
	{ "sp" ,required_argument, NULL, OPTION_SP },
	{ "tcb" ,required_argument, NULL, OPTION_TCB },
	{ "event" ,required_argument, NULL, OPTION_EVENT },
	{ "go" ,no_argument, NULL, OPTION_GO },
	{ "console" ,no_argument, NULL, OPTION_CONSOLE },
	{ "address" ,required_argument, NULL, OPTION_ADDRESS },
	{ "length" ,required_argument, NULL, OPTION_LENGTH },
	{ "upload" ,no_argument, NULL, OPTION_UPLOAD },
	{ "download" ,no_argument, NULL, OPTION_DOWNLOAD },
	{ "port" ,required_argument, NULL, OPTION_PORT },
#if defined(linux)
	{ "realtime" ,no_argument, NULL, OPTION_REALTIME },
#endif
	{ "quiet" ,no_argument, NULL, OPTION_QUIET },
	{ "help" ,no_argument, NULL, OPTION_HELP },
	{ "verbose" ,required_argument, NULL, OPTION_VERBOSE },
	{ "version" ,no_argument, NULL, OPTION_VERSION },
	{ "info" ,no_argument, NULL, OPTION_INFO },
	{ "slot" ,required_argument, NULL, OPTION_SLOT },
	{ "backup" ,no_argument, NULL, OPTION_BACKUP },
	{ "restore" ,no_argument, NULL, OPTION_RESTORE },
	{ "list" ,no_argument, NULL, OPTION_LIST },
	{ "format" ,no_argument, NULL, OPTION_FORMAT },
	{ "delete" ,no_argument, NULL, OPTION_DELETE },
	{ "kanji" ,no_argument, NULL, OPTION_KANJI },
	{ "vram" ,required_argument, NULL, OPTION_VRAM },
	{ "left" ,required_argument, NULL, OPTION_LEFT },
	{ "top" ,required_argument, NULL, OPTION_TOP },
	{ "width" ,required_argument, NULL, OPTION_WIDTH },
	{ "height" ,required_argument, NULL, OPTION_HEIGHT },
	{ "script", required_argument, NULL, OPTION_SCRIPT },
	{ "exec", required_argument, NULL, OPTION_EXEC },
	{ "flash", required_argument, NULL, OPTION_FLASH },
	{ "boot" ,no_argument, NULL, OPTION_BOOT },
	{ "nice" ,required_argument, NULL, OPTION_NICE },
	{ "bpp" ,required_argument, NULL, OPTION_BPP },
	{ "reset" ,no_argument, NULL, OPTION_RESET },
	{ "call" ,required_argument, NULL, OPTION_CALL },
	{ "libps" ,no_argument, NULL, OPTION_LIBPS },
	{ "yaroze" ,no_argument, NULL, OPTION_YAROZE },
	{ "chroot" ,required_argument, NULL, OPTION_CHROOT },
	{ NULL }
};

static char *option_help[] = {
	"set program counter, default is taken from executable",
	"set global pointer, default is taken from executable",
	"set stack pointer, default is taken from executable",
	"set number of task control blocks, default is 4",
	"set number of events, default is 16",
	"start program",
	"attach console & file server",
	"set address for up-/download",
	"set length for download",
	"upload data (needs address,filename), this is the default",
	"download data (needs address,length,filename)",
	"set I/O port of Commlink (default 0x320)",
#if defined(linux)
	"use realtime scheduler",
#endif
	"be quiet (verbose=0)",
	"print this text",
	"set verbosity level (normal=1)",
	"print binary version",
	"print various informations",
	"use this memory card slot (default 1)",
	"backup contents of memory card",
	"restore contents of memory card",
	"list contents of memory card",
	"format memory card",
	"delete entry from memory card",
	"allow kanji printing",
	"up-/download VRAM",
	"set left position in VRAM (default 0)",
	"set top position in VRAM (default 0)",
	"set width of VRAM (default 1024)",
	"set height of VRAM (default 512)",
	"execute a script",
	"execute a binary (on the PSX)",
	"flash ROM",
	"boot the PSX",
	"set nice level of process",
	"set bit per pixel for VRAM operation (16,24 bit)",
	"reset to caetla menu before processing commands",
	"call an address",
	"patch to libps.exe",
	"patch to NetYaroze EXEC",
	"set chroot path for file server",
};

static char *version_text =
#include "VERSION"
;

static char *copyright_text =
"%s - PSX PIO console utility for linux\n"
"Copyright (C) 1997,1998,1999 by \"The PIOCONS Team\";\n"
#include "AUTHORS"
"All Rights Reserved.\n"
"\n";

///////////////////////////////////////////////////////////////////////////

// see defs.h
char *program_name = 0;
u_short verbose_level = 1;
u_short port = 0x320;
u_long flags = 0x0;
int mode = MODE_NORMAL;
uid_t user_1;
uid_t user_2;

///////////////////////////////////////////////////////////////////////////

extern rom_module_t caetla_rom;
extern rom_module_t ezoray_rom;

extern pio_format_t cpe_pio;
extern pio_format_t bfd_pio;
extern pio_format_t psx_pio;

extern vram_format_t tim_vram;
extern vram_format_t ppm_vram;
//extern vram_format_t raw_vram;
//extern vram_format_t rgb_vram;
//extern vram_format_t png_vram;
//extern vram_format_t bmp_vram;

///////////////////////////////////////////////////////////////////////////

static int progress_max;
static char progress_str[100];
static int progress_cur;

void progress_init (int maxlen)
{
	int i;
	progress_max = maxlen;
	progress_cur = 0;
	for (i=0;i<70;i++) progress_str[i] = '-';
	progress_str[70]=0;
}

void progress_bar (int k)
{
	float perc = (k*100.0 / progress_max);
	int cnt = (70*perc) / 100;
	int i;

	if (progress_cur < cnt)
	{
		for (i=progress_cur;i<cnt;i++) progress_str[i] = '#';
		progress_cur = cnt;
		verbosef (VERBOSE_PROGRESS,"\r%3d%% [%s]", (int) perc,progress_str);
	}

}

void progress_end (void)
{
	int i;
	for (i=0;i<70;i++) progress_str[i] = '#';
	verbosef (VERBOSE_PROGRESS,"\r100%% [%s]\n", progress_str);
}

///////////////////////////////////////////////////////////////////////////

void file_server (char *path)
{
	if (path)
	{
		setuid (0);
		if (chroot(path) < 0)
		{
			perror("chroot");
			return;
		}

		setuid (USER_WITH_NO_ACCESS);
		if (chdir("/") < 0)
		{
			perror("chdir");
			return;
		}
	}

	rom_call (server);
}

///////////////////////////////////////////////////////////////////////////

int main (int argc, char **argv)
{
	volatile int opt, i,j;
#if defined(linux)
	struct sched_param scpa;
#endif
	int pid = getpid();
	char *filename = NULL;
	char *chroot_path = NULL;
	char tmp_str [100];
	int nice_level = 0;

	char *rom_filename = NULL;

	char *card_filename = NULL;
	u_short card_slot = 1;
	u_short card_op = MCOP_LIST;

	char *vram_filename = NULL;
	u_short vram_x = 0xffff;
	u_short vram_y = 0xffff;
	u_short vram_w = 0xffff;
	u_short vram_h = 0xffff;
	u_short vram_f = 0xffff;

	u_long _pc = 0x0;
	u_long _gp = 0x0;
	u_long _sp = 0x0;

	pio_file_t *file;
	ps_header_t psh;
	ps_env_t pse = {
		4, 16, 0x801fff00		// tcb,event,stack
	};

	u_long address = 0x0;
	u_long length = 0x0;

	user_1 = getuid();
	user_2 = geteuid();

	setuid (USER_WITH_NO_ACCESS);

	memset (&psh,0,sizeof(ps_header_t));

	program_name = strdup (argv[0]);

	// add ROM plugins (do not change calling sequence !)
//	rom_add_module (&ezoray_rom);
	rom_add_module (&caetla_rom);

	// add EXE format plugins
	pio_add_format (&cpe_pio);
	pio_add_format (&bfd_pio);
	pio_add_format (&psx_pio);

	// add VRAM format plugins
	vram_add_format (&tim_vram);
	vram_add_format (&ppm_vram);

	/*
	**	build the option-char string from the option struct.
	*/

	for (i=j=0; long_options[i].name; i++)
	{
		switch (long_options[i].has_arg)
		{
			case no_argument:
				tmp_str[j++] = (char) long_options[i].val;
				break;
			case required_argument:
				tmp_str[j++] = (char) long_options[i].val;
				tmp_str[j++] = ':';
				break;
		}
	}
	tmp_str[j] = 0;

	while ((opt = getopt_long(argc,argv,tmp_str,long_options,NULL)) != EOF)
	{
		switch (opt)
		{
			case OPTION_HELP:
				verbosef (VERBOSE_ALL,"Usage: %s [options] <filename>\n",program_name);
				for (i=0; long_options[i].name; i++)
				{
					switch (long_options[i].has_arg)
					{
						case no_argument:
							verbosef (VERBOSE_ALL,"--%s,-%c \t%s\n",long_options[i].name,long_options[i].val,option_help[i]);
							break;
						case required_argument:
							verbosef (VERBOSE_ALL,"--%s=x,-%c\t%s\n",long_options[i].name,long_options[i].val,option_help[i]);
							break;
					}
				}
				exit (EXIT_SUCCESS);

			case OPTION_VERSION:
				verbosef (VERBOSE_ALL,"%s version %s\n",program_name,version_text);
				exit (EXIT_SUCCESS);

			case OPTION_PORT:
				port = strtoul (optarg,NULL,16);
				assertf (rangeCheck(0x300,port,0x330),"illegal I/O port 0x%03x",port);
				break;

			case OPTION_QUIET:
				verbose_level = 0;
				break;

			case OPTION_VERBOSE:
				verbose_level = strtoul (optarg,NULL,0);
				break;

			case OPTION_REALTIME:
				flags |= FLAG_REALTIME;
				break;

			case OPTION_CONSOLE:
				flags |= FLAG_CONSOLE;
				break;

			case OPTION_PC:
				_pc = strtoul (optarg,NULL,16);
				break;

			case OPTION_GP:
				_gp = strtoul (optarg,NULL,16);
				break;

			case OPTION_SP:
				pse.stack = strtoul (optarg,NULL,16);
				_sp = pse.stack;
				break;

			case OPTION_TCB:
				pse.task = strtoul (optarg,NULL,0);
				break;

			case OPTION_EVENT:
				pse.event = strtoul (optarg,NULL,0);
				break;

			case OPTION_GO:
				flags |= FLAG_GO;
				break;

			case OPTION_ADDRESS:
				address = strtoul (optarg,NULL,16);
				break;

			case OPTION_LENGTH:
				length = strtoul (optarg,NULL,0);
				break;

			case OPTION_UPLOAD:
				flags &=~ FLAG_DOWNLOAD;
				break;

			case OPTION_DOWNLOAD:
				flags |= FLAG_DOWNLOAD;
				break;

			case OPTION_INFO:
				flags |= FLAG_INFO;
				break;

			case OPTION_SLOT:
				card_slot = strtoul(optarg,NULL,0);
				break;

			case OPTION_BACKUP:
				mode = MODE_MEMORYCARD;
				card_op = MCOP_BACKUP;
				flags |= FLAG_RESET;
				break;

			case OPTION_RESTORE:
				mode = MODE_MEMORYCARD;
				card_op = MCOP_RESTORE;
				flags |= FLAG_RESET;
				break;

			case OPTION_LIST:
				mode = MODE_MEMORYCARD;
				card_op = MCOP_LIST;
				flags |= FLAG_RESET;
				break;

			case OPTION_FORMAT:
				mode = MODE_MEMORYCARD;
				card_op = MCOP_FORMAT;
				flags |= FLAG_RESET;
				break;

			case OPTION_DELETE:
				mode = MODE_MEMORYCARD;
				card_op = MCOP_DELETE;
				flags |= FLAG_RESET;
				break;

			case OPTION_KANJI:
				flags |= FLAG_KANJI;
				break;

			case OPTION_BOOT:
				mode = MODE_BOOT;
				break;

			case OPTION_VRAM:
				mode = MODE_IMAGE;
				vram_filename = optarg;
				break;

			case OPTION_LEFT:
				mode = MODE_IMAGE;
				vram_x = strtoul (optarg,NULL,0);
				break;

			case OPTION_TOP:
				mode = MODE_IMAGE;
				vram_y = strtoul (optarg,NULL,0);
				break;

			case OPTION_WIDTH:
				mode = MODE_IMAGE;
				vram_w = strtoul (optarg,NULL,0);
				break;

			case OPTION_HEIGHT:
				mode = MODE_IMAGE;
				vram_h = strtoul (optarg,NULL,0);
				break;

			case OPTION_BPP:
				mode = MODE_IMAGE;
				vram_f = strtoul (optarg,NULL,0);
				break;

			case OPTION_EXEC:
			//	string = strdup (optarg);
				break;

			case OPTION_SCRIPT:
			//	string = strdup (optarg);
				break;

			case OPTION_FLASH:
				mode = MODE_FLASH;
				rom_filename = optarg;
				flags |= FLAG_RESET;
				break;

			case OPTION_NICE:
				flags |= FLAG_NICE;
				nice_level = strtoul (optarg,NULL,0);
				break;

			case OPTION_RESET:
				flags |= FLAG_RESET;
				break;

			case OPTION_LIBPS:
				flags |= FLAG_LIBPS;
				break;

			case OPTION_YAROZE:
				flags |= FLAG_YAROZE;
				break;

			case OPTION_CHROOT:
				chroot_path = optarg;
				break;

			default:
				errorf ("parsing aborted\n");
				break;
		}
	}

	if (optind < argc) filename = argv[optind];

	// copyright text only for other ppl

	gethostname (tmp_str,256);
	if (strcmp("balster",tmp_str) && strcmp("donald",tmp_str) && strcmp("nag.blackbag.org",tmp_str))
		verbosef (VERBOSE_ALL,copyright_text,program_name);

	if (argc == 1) {
		verbosef(VERBOSE_ALL, "Usage: %s [options] <filename>\n",program_name);
		verbosef(VERBOSE_ALL, "Try '%s --help' for more information.\n\n", program_name);
		return EXIT_SUCCESS;
	}
	// a simple check for 'kterm', force with --kanji !

	if (!strncmp(getenv("TERM"),"kterm",5)) flags |= FLAG_KANJI;

	// allocate I/O ports

	errno = 0;
	verbosef (VERBOSE_RSRC,"Allocating I/O ports at 0x%03x.\n",port);
#if defined(linux)
	if (ioperm (port,3,1) == -1)
#endif
#if defined(__NetBSD__)
	if (i386_iopl(1) == -1)
#endif
#if defined(__FreeBSD__)
	if (open("/dev/io", O_RDONLY) < 0)
#endif
		errorf ("no access to ports at 0x%03x; %s\n", port, strerror (errno));

#if defined(linux)
	// allow 100% CPU blocking

	if (flags & FLAG_REALTIME)
	{
		sched_getparam (pid,&scpa);
		scpa.sched_priority = 1;
		verbosef (VERBOSE_RSRC,"Changing scheduler\n");
		if (sched_setscheduler (pid,SCHED_FIFO,&scpa) == -1)
			errorf ("can't change scheduler; %s\n", strerror (errno));
	}
#endif

	// set nice level

	if (flags & FLAG_NICE) nice (nice_level);

	rom_detect();		// now we can detect the rom
	if (flags & FLAG_RESET)
		rom_call (reset);	// only reset if needed.

	// print out default settings if wanted

	if (flags & FLAG_INFO)
	{
		verbosef (VERBOSE_ALL,
			"ROM/Version: %s/%s\n"
			"Description: %s\n"
			"Copyright  : %s\n"
			"Author     : %s\n"
			"Filename   : %s\n"
			"Direction  : %s\n"
			"Address    : 0x%08lx\n"
			"Length     : 0x%08lx\n"
			"PC/GP/SP   : 0x%08lx/0x%08lx/0x%08lx\n"
			"Env:Task   : %d\n"
			"Env:Event  : %d\n"
			"VRAM Area  : %dx%d+%d+%d %d-Bit\n"
			"Card Slot  : %d\n"
			"Card Action: %s\n\n"
			, rom->name, rom->version
			, rom->description, rom->copyright, rom->author
			, filename ? filename : ""
			, (flags & FLAG_DOWNLOAD) ? "DOWNLOAD" : "UPLOAD"
			, address , length
			, _pc, _gp, _sp
			, (int) pse.task, (int) pse.event
			, vram_w, vram_h, vram_x, vram_y, vram_f
			, card_slot, card_name_of_op(card_op)
		);
		
		verbosef (VERBOSE_ALL,"EXE Plugins: ");
		pio_list();
		verbosef (VERBOSE_ALL,"IMG Plugins: ");
		vram_list();
		verbosef (VERBOSE_ALL,"ROM Plugins: ");
		rom_list();
		verbosef (VERBOSE_ALL,"\n");
	}

	switch (mode)
	{
		case MODE_CALL:
			rom_call(call, address);
			break;
		case MODE_NORMAL:
			if (flags & FLAG_DOWNLOAD)
			{
				if (filename==NULL)
				{
					filename = "/dev/stdout";
					warnf ("filename missing; using \"%s\"\n",filename);
				}
			
				file = pio_file_openw (filename,length);
				if (file)
				{
					verbosef (VERBOSE_PROGRESS,"Downloading binary from 0x%08lx..0x%08lx (%ld bytes) to \"%s\"\n",
						address,address+file->length,file->length,filename);
					rom_call (recv_memory,address,file->buffer,file->length);
					pio_file_close(file);
				}
				else errorf ("download aborted\n");
			}
			else
			{
				pio_format_t *pio;
		
				if (filename==NULL) {
					if ((flags & FLAG_CONSOLE) == 0 && (flags & FLAG_RESET) == 0)
						warnf ("filename missing\n");
				}
				else
				{
					file = pio_file_openr (filename);
					if (file)
					{
						// if an address was specified, treat file as binary upload

						if (address)
						{
							verbosef (VERBOSE_PROGRESS,"Uploading binary to 0x%08lx..0x%08lx (%ld bytes) from \"%s\"\n",
								address,address+file->length,file->length,filename);
							rom_call (send_memory,address,file->buffer,file->length);
						}
						else
						{
							pio = pio_detect(file);
							if (pio)
							{
								if (flags & FLAG_INFO)
								{
									pio->dump(file);
								}
								//else
								{
									pio->send(file);
									pio->get_header(file,&psh);
								}
							}
							else errorf ("binary format not recognized\n");
						}

						pio_file_close (file);
					}
					else errorf ("upload aborted\n");
				}

				if (flags & FLAG_GO)
				{
					if (_pc) psh.pc0 = _pc;		// override gp,pc if wanted.
					if (_gp) psh.gp0 = _gp;

					rom_call (go,&pse,&psh,(bool)(flags & FLAG_CONSOLE));
				}
				
				if (flags & FLAG_CONSOLE)
					file_server (chroot_path);
			}
		break;

		case MODE_BOOT:
			psh.pc0 = 0xbfc00000;
			rom_call (go,&pse,&psh,(bool)(flags & FLAG_CONSOLE));
			if (flags & FLAG_CONSOLE)
				file_server (chroot_path);
			break;

		case MODE_MEMORYCARD:
			if (optind >= argc)
			{
				card_operation (card_slot, card_op, NULL);
			}
			else while (optind < argc)
			{
				card_filename = argv[optind];
				card_operation (card_slot, card_op, card_filename);
				optind++;
			}
			break;

		case MODE_IMAGE:
			vram_operation (vram_x,vram_y,vram_w,vram_h,vram_f,vram_filename,(bool) !(flags & FLAG_DOWNLOAD));
			break;

		case MODE_FLASH:
			if (rom_filename==NULL) errorf ("ROM filename missing\n");

			file = pio_file_openr (rom_filename);
			if (file)
			{
				rom_call (flash,file->buffer,file->length);
				pio_file_close (file);
			}
			else errorf ("flash aborted\n");
			break;

		default:
			// unsupported
			break;
	}

	return EXIT_SUCCESS;
}
