/* $Id: ppm.c,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#include "vram.h"
#include "error.h"
#include "pio.h"
#include "getint.h"

static bool ppm_detect (char *filename)
{
	char *s = rindex(filename,'.');

	if (s==0) return FALSE;

	if (!strcasecmp(s,".ppm")) return TRUE;

	return FALSE;
}

static void ppm_write (char *filename, vram_image_t *image)
{
	FILE *file;
	int i;

	file = fopen (filename,"wb");
	if (file)
	{
		if (image->f==1)
		{
			u_char *p = (u_char *) image->pixel;
			u_char r,g,b;
			
			fprintf(file,
				"P6\n"
				"# file written by piocons !\n"
				"%d %d\n"
				"255\n"
				,image->w,image->h);

			for (i=0;i<(image->w*image->h);i++)
			{
				b = *p++; r = *p++; g = *p++;
				fputc (r,file);
				fputc (g,file);
				fputc (b,file);
			}
		}
		else
		{
			#define RedOfPixel(pixel)	(((pixel)&(0x7c00))>>10)
			#define GreenOfPixel(pixel)	(((pixel)&(0x03e0))>>5)
			#define BlueOfPixel(pixel)	(((pixel)&(0x001f))>>0)

			fprintf(file,
				"P6\n"
				"# file written by piocons !\n"
				"%d %d\n"
				"31\n"
				,image->w,image->h);

			for (i=0;i<(image->w*image->h);i++)
			{
				fputc(BlueOfPixel(image->pixel[i]),file);
				fputc(GreenOfPixel(image->pixel[i]),file);
				fputc(RedOfPixel(image->pixel[i]),file);
			}
		}

		fclose (file);
	}
	else errorf("vram download aborted\n");
}

static void ppm_read (char *filename, vram_image_t *image)
{
	FILE *file;
	int i,w,h;

	file = fopen (filename,"rb");
	if (file)
	{
		char ppmid[2];
		u_char *p;
	
		ppmid[0] = fgetc(file);
		ppmid[1] = fgetc(file);
	
		if (((ppmid[0]=='P') && (ppmid[1]=='3' || ppmid[1]=='6')))
		{
			yyrestart (file);

			image->f = 1;
			image->x = 0;
			image->y = 0;
			w = yylex();
			h = yylex();
			image->w = (w/2)*3;
			image->h = h;
			p = (u_char *) image->pixel = malloc (w*h*3);
			i = yylex();			// depth. not used yet

			printf ("%d %d %d\n",w,h,i);
			
			if (ppmid[1]=='3')
			{
				for (i=0;i<(w*h*3);i++) p[i] = yylex();
			}
			else
			{
				for (i=0;i<(w*h*3);i++) p[i] = fgetc(file);
			}
		}
		else errorf("input file not P3/P6 ppm file\n");

		fclose (file);
	}
	else errorf("vram upload aborted\n");
}

vram_format_t ppm_vram = {
	NULL,
	
	// module informations	
	"PPM",
	"0.1",
	"ppm image driver",
	"Daniel Balster <dbalster@uni-paderborn.de>",
	"Daniel Balster <dbalster@uni-paderborn.de>",

	ppm_detect,
	ppm_write,
	ppm_read,
};
