/* $Id: psx.c,v 1.2 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************

	PS-X EXE file format.

 */

#include "pio.h"
#include "rom.h"
#include "error.h"
#include "libps.h"

static bool psx_detect (pio_file_t *f)
{
	psx_header_t *p = (psx_header_t *) f->buffer;

	if (!strncmp(p->key,"PS-X EXE",8)) return TRUE;

	return FALSE;
}

static bool psx_send (pio_file_t *f)
{
	psx_header_t *p = (psx_header_t *) f->buffer;
	u_char *bytes = (u_char *) p;
	u_long addr,size;

	assertz (f,"file");
	assertz (p,"buffer");

	addr = p->exec.t_addr;
	size = p->exec.t_size;

	bytes += p->text ? p->text : 2048;	// For SCE format

	verbosef (VERBOSE_ALL,
		"Object file format is \"PS-X EXE\"\n"
		"PC=0x%08lx GP=0x%08lx\n"
		,p->exec.pc0,p->exec.gp0);
	verbosef (VERBOSE_PROGRESS,
		"Section [   .text] Address [%08lx] Length [%06lx] *\n"
		"Section [   .data] Address [%08lx] Length [%06lx]\n"
		"Section [    .bss] Address [%08lx] Length [%06lx]\n"
		,p->exec.t_addr,p->exec.t_size
		,p->exec.d_addr,p->exec.d_size
		,p->exec.b_addr,p->exec.b_size);

	rom_call (send_memory,addr,bytes,size);

	if (flags & FLAG_LIBPS)
	{
		verbosef (VERBOSE_ALL, "Patch to libps.exe\n");
		rom_call (send_memory, (u_long)LIBPS_PATCH1_ADDR,
			(u_char *)libps_patch_1, sizeof (libps_patch_1));
		rom_call (send_memory, (u_long)LIBPS_PATCH2_ADDR,
			(u_char *)libps_patch_2, sizeof (libps_patch_2));
	}

	return TRUE;
}

static bool psx_dump (pio_file_t *f)
{
	psx_header_t *p = (psx_header_t *) f->buffer;

	assertz (f,"file");

	verbosef (VERBOSE_ALL,
	"Title: \"%s\"\n"
	"PC0:0x%08lx GP0:0x%08lx\n"
	"TXT:0x%08lx DAT:0x%08lx\n"
	".text:0x%08lx, %d\n"
	".data:0x%08lx, %d\n"
	" .bss:0x%08lx, %d\n"
	"stack:0x%08lx, %d\n",
	p->title,
	p->exec.pc0,p->exec.gp0,
	p->text,p->data,
	p->exec.t_addr,(int) p->exec.t_size,
	p->exec.d_addr,(int) p->exec.d_size,
	p->exec.b_addr,(int) p->exec.b_size,
	p->exec.s_addr,(int) p->exec.s_size);

	return TRUE;
}

static bool psx_get_header (pio_file_t *f, ps_header_t *h)
{
	psx_header_t *p = (psx_header_t *) f->buffer;

	assertz (f,"file");
	assertz (h,"header");

	memcpy (h,&(p->exec),sizeof(ps_header_t));

	return TRUE;
}

pio_format_t psx_pio = {
	NULL,
	
	// module informations	
	"PS-X EXE",
	"0.1",
	"PS-X EXE format driver",
	"Daniel Balster <dbalster@uni-paderborn.de>",
	"Daniel Balster <dbalster@uni-paderborn.de>",

	NULL,
	psx_detect,
	psx_send,
	psx_get_header,
	psx_dump,
};
