/* $Id: rom.c,v 1.4 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#include "rom.h"
#include "error.h"

u_long flasher_rom[] = {
#include "flasher.h"
};

static rom_module_t *rom_root;
rom_module_t *rom = NULL;

bool rom_error;

/*
**	we assume that there is at least one module at the rom_root.
*/

void rom_add_module (rom_module_t *m)
{
	m->next = rom_root;
	rom_root = m;
}

/*
**	we assume that there is at least one module at the rom_root.
*/

void rom_detect (void)
{
	rom = rom_root;
	
	verbosef (VERBOSE_CHECK,"Autodetecting ROM:\n");

	do {
		verbosef (VERBOSE_CHECK,"%s: ",rom->name);
		if (rom->detect()) break;
		verbosef (VERBOSE_CHECK,"NO\n");
		rom = rom->next;
	} while (rom != NULL);

	if (rom_error) errorf ("No communication possible!\n");

	if (rom) verbosef (VERBOSE_CHECK,"YES\n");

	if (rom==NULL) errorf ("unknown/unsupported ROM!\n");
}

void rom_list (void)
{
	rom_module_t *rom;

	rom = rom_root;
		
	do {
		verbosef (VERBOSE_ALL,"\"%s\" ",rom->name);
		rom = rom->next;
	} while (rom != NULL);

	verbosef (VERBOSE_ALL,"\n");
}

u_char rom_swap_8 (u_char x)
{
	volatile int try = 0;
	rom_error = FALSE;

	setuid (USER_WITH_IO_ACCESS);

#if defined(linux)
	outb (x,PORT_RW);
#else
	outb (PORT_RW,x);
#endif
	while ( inb(PORT_STATUS) & 1 )
	{
		try++;
		if (try > ROM_TIMEOUT)
		{
			rom_error = TRUE;
			break;
		}
	}
	x = inb (PORT_RW);

	setuid (USER_WITH_NO_ACCESS);

	return x;
}

u_short rom_swap_16 (u_short x)
{
        return ( rom_swap_8(x>>8) << 8 | rom_swap_8(x) );
}

u_long rom_swap_32 (u_long x)
{
  	return ( rom_swap_16(x>>16) << 16 | rom_swap_16(x) );
}

void rom_put_string (char *x)
{
	while (*x) rom_swap_8(*x++);
}

void rom_get_string (char *x)
{
	while ((*x++ = rom_swap_8(0)));
}

u_char rom_send_memory (void *buf, u_long size)
{
	u_char *p=buf, chk=0, ck;
	u_long c;

	for (c=0;c<size;c++,chk+=ck) rom_swap_8 (ck=*p++);

	return chk;
}

u_char rom_recv_memory (void *buf, u_long size)
{
	u_char *p=buf, chk=0, ck;
	u_long c;

	for (c=0;c<size;c++,chk+=ck) *p++ = (ck = rom_swap_8(0));

	return chk;
}
