/* $Id: rom.h,v 1.2 1999/03/28 05:33:13 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

#ifndef __ROM_H__
#define __ROM_H__

#include "defs.h"
#include "vram.h"
#include "card.h"

#define ROM_TIMEOUT 200000

#define ROM_HAS_SENDMEM		(1L <<  0)		// memory support
#define ROM_HAS_RECVMEM		(1L <<  1)
#define ROM_HAS_SENDVRAM	(1L <<  2)		// vram support
#define ROM_HAS_RECVVRAM	(1L <<  3)
#define ROM_HAS_SENDWAVE	(1L <<  4)		// wave support
#define ROM_HAS_RECVWAVE	(1L <<  5)
#define ROM_HAS_BOOT		(1L <<  6)		// boot executable support
#define ROM_HAS_SERVER		(1L <<  7)		// file server support
#define ROM_HAS_FLASH		(1L <<  8)		// flash support
#define ROM_HAS_RESUME		(1L <<  9)		// resume support
#define ROM_HAS_FREEZE		(1L << 10)		// freezer support
#define ROM_HAS_MEMCARD		(1L << 11)		// memorycard support
#define ROM_HAS_BREAKPOINTS	(1L << 12)		// breakpoint support
#define ROM_HAS_CHEATS		(1L << 13)		// cheating support

typedef struct _rom_module_t {

	struct _rom_module_t *next;

	char *name;				// "CAETLA"
	char *version;			// "0.1"
	char *description;		// "caetla rom driver"
	char *copyright;		// "Daniel Balster <dbalster@uni-paderborn.de>"
	char *author;			// "Daniel Balster <dbalster@uni-paderborn.de>"

	u_long flags;			// flags. ROM_HAS_... currently unused!

	// Communicate with ROM, return TRUE if ROM is supported by this module.
	bool (*detect) (void);

	// reset PSX (after this call the communication should be possible)
	void (*reset) (void);

	// boot/exec playstation executable. if filename==NULL, try to boot
	// with "cdrom:\system.cnf" ! if ``console'' is true redirect console
	void (*boot) (char *filename, ps_env_t *e, bool console);

	// send ``size'' bytes from ``buffer'' to ``address'' in the PSX.
	void (*send_memory) (u_long address, u_char *buffer, u_long size);

	// recv ``size'' bytes to ``buffer'' from ``address'' in the PSX.
	void (*recv_memory) (u_long address, u_char *buffer, u_long size);

	// send vram image to PSX
	void (*send_vram) (vram_image_t *image);

	// receive vram image from PSX ("grab")
	void (*recv_vram) (vram_image_t *image);

	// send wave to playstation
	void (*send_wave) (u_long address, u_char *buffer, u_long size);

	// receive wave from playstation
	void (*recv_wave) (u_long address, u_char *buffer, u_long size);

	// run program in memory
	// 
	void (*go) (ps_env_t *e, ps_header_t *h, bool console);

	// call an address
	void (*call) (unsigned long address);

	// start console and file server
	void (*server) (void);

	// flash content of buffer into flashrom of AR
	void (*flash) (u_char *buffer, u_long size);

	void (*card_on) (void);
	void (*card_off) (void);
	void (*card_list) (u_short slot,card_list_t *list);
	void (*card_backup) (u_short slot,u_short file, u_char *content);
	void (*card_restore) (u_short slot,u_short file, u_char *content, u_long size);

	void (*delete_card)(u_short slot, u_short file);
	void (*format_card)(u_short slot);
//	bool (*write_card)(u_short slot, u_char *buffer, u_long size);
//	bool (*rename_card)(u_long address, u_char *buffer, u_long size);
//	bool (*create_card)(u_long address, u_char *buffer, u_long size);

} rom_module_t;

/*
	use "rom_call (function,...)" to get slot checking and error generation.
*/

#define rom_call(function,args...) \
	{\
		if (rom->function==NULL) \
			errorf ("ROM driver \"%s\" does not support " #function "()\n",rom->name); \
		else rom->function ( args );\
	}

extern rom_module_t *rom;
extern u_long flasher_rom[];
extern bool rom_error;

/** prototypes **/

void rom_add_module (rom_module_t *m);
void rom_detect (void);
u_char rom_swap_8 (u_char x);
void rom_list (void);
u_short rom_swap_16 (u_short x);
u_long rom_swap_32 (u_long x);
void rom_put_string (char *x);
void rom_get_string (char *x);
u_char rom_send_memory (void *buf, u_long size);
u_char rom_recv_memory (void *buf, u_long size);

#endif  __ROM_H__
