/* $Id: script.l,v 1.1.1.1 1999/01/30 06:09:06 andrewk Exp $
 ******************************************************************************
 * 
 * piocons - PSX PIO console utility for linux
 * Copyright (C) 1997,1998 by "The PIOCONS Team" (see AUTHORS)
 * All Rights Reserved.
 * 
 ******************************************************************************
 */

%option noyywrap
%option stack

#include "rom.h"
#include "error.h"

%{
#include <stdio.h>
#include <getopt.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <time.h>		/* for nanosleep(), usleep() */
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#include "main.h"
#include "bfd.h"
#include "send.h"


void process_script (FILE *f)
{
	yyin = f;
	yylex();
}

%}

WS					[ \t\n]*
VARIABLE			[A-Za-z_0-9]+
STRING				[^\n\"\$]*
HEXVAL				"0x"[0-9a-fA-F]{1,8}
DECVAL				[1-9][0-9]*

%x COMMENT
%x INTERN
%x LOCAL
%x PSX

%x UPLOAD_BFD
%x UPLOAD_CPE
%x UPLOAD_BIN
%x UPLOAD_BIN2

%%

<*>"#"		yy_push_state(COMMENT);

{WS}		/* */
"intern"{WS}	yy_push_state(INTERN);
"local"{WS}		yy_push_state(LOCAL);
"psx"{WS}		yy_push_state(PSX);

<COMMENT>[^\n]*		/* ignore */
<COMMENT>\n			yy_pop_state();

<INTERN>"upload_bfd"{WS}	yy_push_state(UPLOAD_BFD);
<INTERN>"upload_cpe"{WS}	yy_push_state(UPLOAD_CPE);
<INTERN>"upload_bin"{WS}	yy_push_state(UPLOAD_BIN2);
<INTERN>"go"				{ psx_go(global.address); yy_pop_state(); }
<INTERN>"console"			{ psx_output(0); yy_pop_state(); }
<INTERN>"waitprompt"		{ psx_output(1); yy_pop_state(); }
<INTERN>"break"				{ psx_break(); yy_pop_state(); }
<INTERN>.					{ printf("error!\n"); yy_pop_state(); }

<UPLOAD_BFD>[^\n]*				{ uploadfile_bfd(yytext); yy_pop_state(); }
<UPLOAD_CPE>[^\n]*				{ uploadfile_cpe(yytext); yy_pop_state(); }
<UPLOAD_BIN>[^\n]*				{ uploadfile_bin(yytext); yy_pop_state(); }

<UPLOAD_BIN2>{HEXVAL}{WS}		{ global.address = strtoul(yytext,NULL,16); BEGIN(UPLOAD_BIN); }

<LOCAL>[^\n]*					{ system(yytext); yy_pop_state(); }
<PSX>[^\n]*						{ psx_command(yytext); psx_output(1); yy_pop_state(); }
